use axum::{
    extract::{Path, State},
    http::StatusCode,
    response::Json,
};
use serde::{Deserialize, Serialize};
use std::sync::Arc;
use tokio::sync::Mutex;

use crate::blockchain::{Blockchain, Transaction};
use crate::wallet::Wallet;

pub type SharedBlockchain = Arc<Mutex<Blockchain>>;

#[derive(Serialize)]
pub struct BalanceResponse {
    pub address: String,
    pub balance: f64,
}

#[derive(Serialize)]
pub struct BlockchainInfo {
    pub total_blocks: usize,
    pub difficulty: usize,
    pub mining_reward: f64,
    pub pending_transactions: usize,
}

#[derive(Deserialize)]
pub struct SendTransactionRequest {
    pub from_private_key: String,
    pub to: String,
    pub amount: f64,
    pub atproto_did: Option<String>,
}

#[derive(Serialize)]
pub struct TransactionResponse {
    pub success: bool,
    pub message: String,
    pub transaction_hash: Option<String>,
}

#[derive(Serialize)]
pub struct WalletResponse {
    pub address: String,
    pub public_key: String,
    pub atproto_did: Option<String>,
}

pub async fn get_balance(
    Path(address): Path<String>,
    State(blockchain): State<SharedBlockchain>,
) -> Json<BalanceResponse> {
    let blockchain = blockchain.lock().await;
    let balance = blockchain.get_balance(&address);
    Json(BalanceResponse { address, balance })
}

pub async fn get_blockchain_info(
    State(blockchain): State<SharedBlockchain>,
) -> Json<BlockchainInfo> {
    let blockchain = blockchain.lock().await;
    Json(BlockchainInfo {
        total_blocks: blockchain.chain.len(),
        difficulty: blockchain.difficulty,
        mining_reward: blockchain.mining_reward,
        pending_transactions: blockchain.pending_transactions.len(),
    })
}

pub async fn create_wallet(atproto_did: Option<String>) -> Json<WalletResponse> {
    let wallet = Wallet::new(atproto_did);
    Json(WalletResponse {
        address: wallet.address,
        public_key: wallet.public_key,
        atproto_did: wallet.atproto_did,
    })
}

pub async fn send_transaction(
    State(blockchain): State<SharedBlockchain>,
    Json(payload): Json<SendTransactionRequest>,
) -> Result<Json<TransactionResponse>, StatusCode> {
    let wallet = match Wallet::from_private_key(&payload.from_private_key, payload.atproto_did) {
        Ok(w) => w,
        Err(e) => {
            return Ok(Json(TransactionResponse {
                success: false,
                message: format!("Invalid private key: {}", e),
                transaction_hash: None,
            }));
        }
    };

    let mut transaction = Transaction::new(
        wallet.address.clone(),
        payload.to,
        payload.amount,
        wallet.atproto_did,
    );

    if let Some(ref signing_key) = wallet.private_key {
        transaction.sign(signing_key);
        let tx_hash = transaction.calculate_hash();

        let mut blockchain = blockchain.lock().await;
        match blockchain.add_transaction(transaction) {
            Ok(()) => Ok(Json(TransactionResponse {
                success: true,
                message: "Transaction added to pending pool".to_string(),
                transaction_hash: Some(tx_hash),
            })),
            Err(e) => Ok(Json(TransactionResponse {
                success: false,
                message: e,
                transaction_hash: None,
            })),
        }
    } else {
        Ok(Json(TransactionResponse {
            success: false,
            message: "Failed to sign transaction".to_string(),
            transaction_hash: None,
        }))
    }
}

pub async fn mine_block(
    Path(miner_address): Path<String>,
    State(blockchain): State<SharedBlockchain>,
) -> Result<Json<serde_json::Value>, StatusCode> {
    let mut blockchain = blockchain.lock().await;
    
    if blockchain.pending_transactions.is_empty() {
        return Ok(Json(serde_json::json!({
            "success": false,
            "message": "No pending transactions to mine"
        })));
    }

    let pending_count = blockchain.pending_transactions.len();
    blockchain.mine_pending_transactions(miner_address.clone());
    
    Ok(Json(serde_json::json!({
        "success": true,
        "message": "Block mined successfully",
        "transactions_included": pending_count,
        "miner": miner_address
    })))
}