use super::block::Block;
use super::transaction::Transaction;
use serde::{Deserialize, Serialize};
use std::collections::HashMap;

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Blockchain {
    pub chain: Vec<Block>,
    pub difficulty: usize,
    pub pending_transactions: Vec<Transaction>,
    pub mining_reward: f64,
    pub balances: HashMap<String, f64>,
    pub block_time: u64, // Target time between blocks in seconds
    pub difficulty_adjustment_interval: u64, // Number of blocks between difficulty adjustments
}

impl Blockchain {
    pub fn new() -> Self {
        let mut blockchain = Blockchain {
            chain: vec![Block::genesis()],
            difficulty: 2,
            pending_transactions: vec![],
            mining_reward: 50.0,
            balances: HashMap::new(),
            block_time: 30, // 30 seconds per block
            difficulty_adjustment_interval: 10, // Adjust every 10 blocks
        };
        blockchain.balances.insert("genesis".to_string(), 1000000.0);
        blockchain
    }

    pub fn add_transaction(&mut self, transaction: Transaction) -> Result<(), String> {
        if transaction.from == transaction.to {
            return Err("Cannot send to yourself".to_string());
        }

        if transaction.amount <= 0.0 {
            return Err("Amount must be positive".to_string());
        }

        if transaction.from != "genesis" && transaction.from != "system" {
            let balance = self.get_balance(&transaction.from);
            let total_cost = transaction.amount + transaction.fee;
            if balance < total_cost {
                return Err(format!("Insufficient balance. Need {} (amount: {}, fee: {}), have {}", 
                    total_cost, transaction.amount, transaction.fee, balance));
            }
        }

        self.pending_transactions.push(transaction);
        Ok(())
    }

    pub fn mine_pending_transactions(&mut self, mining_reward_address: String) {
        let reward_tx = Transaction::new(
            "system".to_string(),
            mining_reward_address.clone(),
            self.mining_reward,
            None,
        );
        self.pending_transactions.push(reward_tx);

        let mut block = Block::new(
            self.chain.len() as u64,
            self.pending_transactions.clone(),
            self.get_latest_block().hash.clone(),
        );
        block.mine_block(self.difficulty);

        // Calculate total fees for miner
        let mut total_fees = 0.0;
        
        for tx in &block.transactions {
            if tx.from != "system" && tx.from != "genesis" {
                let total_deduction = tx.amount + tx.fee;
                *self.balances.entry(tx.from.clone()).or_insert(0.0) -= total_deduction;
                total_fees += tx.fee;
            }
            *self.balances.entry(tx.to.clone()).or_insert(0.0) += tx.amount;
        }
        
        // Give fees to miner
        if total_fees > 0.0 {
            *self.balances.entry(mining_reward_address.clone()).or_insert(0.0) += total_fees;
        }

        self.chain.push(block);
        self.pending_transactions.clear();
        
        // Adjust difficulty if needed
        self.adjust_difficulty();
    }
    
    fn adjust_difficulty(&mut self) {
        let chain_length = self.chain.len() as u64;
        if chain_length % self.difficulty_adjustment_interval == 0 && chain_length > 0 {
            let start_index = (chain_length - self.difficulty_adjustment_interval) as usize;
            let end_index = chain_length as usize - 1;
            
            let time_taken = self.chain[end_index].timestamp.timestamp() 
                - self.chain[start_index].timestamp.timestamp();
            let expected_time = (self.block_time * self.difficulty_adjustment_interval) as i64;
            
            if time_taken < expected_time / 2 {
                self.difficulty += 1;
                println!("Difficulty increased to {}", self.difficulty);
            } else if time_taken > expected_time * 2 {
                if self.difficulty > 1 {
                    self.difficulty -= 1;
                    println!("Difficulty decreased to {}", self.difficulty);
                }
            }
        }
    }

    pub fn get_balance(&self, address: &str) -> f64 {
        *self.balances.get(address).unwrap_or(&0.0)
    }

    pub fn get_latest_block(&self) -> &Block {
        self.chain.last().unwrap()
    }

    pub fn is_valid(&self) -> bool {
        for i in 1..self.chain.len() {
            let current = &self.chain[i];
            let previous = &self.chain[i - 1];

            if current.hash != current.calculate_hash() {
                return false;
            }

            if current.previous_hash != previous.hash {
                return false;
            }

            if &current.hash[..self.difficulty] != "0".repeat(self.difficulty) {
                return false;
            }
        }
        true
    }
}