use chrono::{DateTime, Utc};
use ed25519_dalek::{Signature, Signer, SigningKey, VerifyingKey};
use serde::{Deserialize, Serialize};
use sha2::{Digest, Sha256};

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Transaction {
    pub from: String,
    pub to: String,
    pub amount: f64,
    pub fee: f64,
    pub timestamp: DateTime<Utc>,
    pub signature: Option<String>,
    pub atproto_did: Option<String>,
}

impl Transaction {
    pub fn new(from: String, to: String, amount: f64, atproto_did: Option<String>) -> Self {
        let fee = Self::calculate_fee(amount);
        Transaction {
            from,
            to,
            amount,
            fee,
            timestamp: Utc::now(),
            signature: None,
            atproto_did,
        }
    }
    
    pub fn calculate_fee(amount: f64) -> f64 {
        // 0.1% fee with minimum of 0.01
        (amount * 0.001).max(0.01)
    }

    pub fn calculate_hash(&self) -> String {
        let data = format!(
            "{}{}{}{}{:?}{:?}",
            self.from, self.to, self.amount, self.fee, self.timestamp, self.atproto_did
        );
        let mut hasher = Sha256::new();
        hasher.update(data.as_bytes());
        hex::encode(hasher.finalize())
    }

    pub fn sign(&mut self, private_key: &SigningKey) {
        let hash = self.calculate_hash();
        let signature = private_key.sign(hash.as_bytes());
        self.signature = Some(hex::encode(signature.to_bytes()));
    }

    pub fn verify_signature(&self, public_key: &VerifyingKey) -> bool {
        if let Some(ref sig_hex) = self.signature {
            if let Ok(sig_bytes) = hex::decode(sig_hex) {
                if let Ok(signature) = Signature::from_slice(&sig_bytes) {
                    let hash = self.calculate_hash();
                    return public_key.verify_strict(hash.as_bytes(), &signature).is_ok();
                }
            }
        }
        false
    }
}