use futures::{prelude::*, select};
use libp2p::{
    gossipsub, identify, mdns, noise, swarm::NetworkBehaviour, swarm::SwarmEvent, tcp, yamux,
    PeerId, Swarm, SwarmBuilder,
};
use std::collections::hash_map::DefaultHasher;
use std::hash::{Hash, Hasher};
use std::time::Duration;
use tokio::sync::mpsc;

use super::message::{MessageType, NetworkMessage};
use crate::blockchain::{Block, Blockchain, Transaction};

#[derive(NetworkBehaviour)]
pub struct AicoinBehaviour {
    pub gossipsub: gossipsub::Behaviour,
    pub mdns: mdns::tokio::Behaviour,
    pub identify: identify::Behaviour,
}

pub struct P2PNetwork {
    swarm: Swarm<AicoinBehaviour>,
    topic: gossipsub::IdentTopic,
    blockchain_tx: mpsc::Sender<MessageType>,
}

impl P2PNetwork {
    pub async fn new(
        blockchain_tx: mpsc::Sender<MessageType>,
        port: u16,
    ) -> Result<Self, Box<dyn std::error::Error>> {
        let mut swarm = SwarmBuilder::with_new_identity()
            .with_tokio()
            .with_tcp(
                tcp::Config::default(),
                noise::Config::new,
                yamux::Config::default,
            )?
            .with_behaviour(|key| {
                let message_id_fn = |message: &gossipsub::Message| {
                    let mut s = DefaultHasher::new();
                    message.data.hash(&mut s);
                    gossipsub::MessageId::from(s.finish().to_string())
                };

                let gossipsub_config = gossipsub::ConfigBuilder::default()
                    .heartbeat_interval(Duration::from_secs(10))
                    .validation_mode(gossipsub::ValidationMode::Strict)
                    .message_id_fn(message_id_fn)
                    .build()
                    .expect("Valid config");

                let gossipsub = gossipsub::Behaviour::new(
                    gossipsub::MessageAuthenticity::Signed(key.clone()),
                    gossipsub_config,
                )
                .expect("Correct configuration");

                let mdns = mdns::tokio::Behaviour::new(
                    mdns::Config::default(),
                    key.public().to_peer_id(),
                )?;

                let identify = identify::Behaviour::new(identify::Config::new(
                    "/aicoin/1.0.0".to_string(),
                    key.public(),
                ));

                Ok(AicoinBehaviour {
                    gossipsub,
                    mdns,
                    identify,
                })
            })?
            .with_swarm_config(|c| c.with_idle_connection_timeout(Duration::from_secs(60)))
            .build();

        swarm.listen_on(format!("/ip4/0.0.0.0/tcp/{}", port).parse()?)?;

        let topic = gossipsub::IdentTopic::new("aicoin-network");
        swarm.behaviour_mut().gossipsub.subscribe(&topic)?;

        Ok(P2PNetwork {
            swarm,
            topic,
            blockchain_tx,
        })
    }

    pub async fn broadcast_block(&mut self, block: Block) {
        let message = NetworkMessage {
            message_type: MessageType::NewBlock(block),
            timestamp: chrono::Utc::now(),
        };
        
        if let Ok(data) = serde_json::to_vec(&message) {
            if let Err(e) = self
                .swarm
                .behaviour_mut()
                .gossipsub
                .publish(self.topic.clone(), data)
            {
                eprintln!("Failed to publish block: {:?}", e);
            }
        }
    }

    pub async fn broadcast_transaction(&mut self, transaction: Transaction) {
        let message = NetworkMessage {
            message_type: MessageType::NewTransaction(transaction),
            timestamp: chrono::Utc::now(),
        };
        
        if let Ok(data) = serde_json::to_vec(&message) {
            if let Err(e) = self
                .swarm
                .behaviour_mut()
                .gossipsub
                .publish(self.topic.clone(), data)
            {
                eprintln!("Failed to publish transaction: {:?}", e);
            }
        }
    }

    pub async fn run(mut self) {
        loop {
            select! {
                event = self.swarm.select_next_some() => {
                    match event {
                        SwarmEvent::Behaviour(AicoinBehaviourEvent::Mdns(mdns::Event::Discovered(list))) => {
                            for (peer_id, _) in list {
                                println!("Discovered peer: {}", peer_id);
                                self.swarm.behaviour_mut().gossipsub.add_explicit_peer(&peer_id);
                            }
                        }
                        SwarmEvent::Behaviour(AicoinBehaviourEvent::Mdns(mdns::Event::Expired(list))) => {
                            for (peer_id, _) in list {
                                println!("Peer expired: {}", peer_id);
                                self.swarm.behaviour_mut().gossipsub.remove_explicit_peer(&peer_id);
                            }
                        }
                        SwarmEvent::Behaviour(AicoinBehaviourEvent::Gossipsub(gossipsub::Event::Message {
                            propagation_source: _,
                            message_id: _,
                            message,
                        })) => {
                            if let Ok(msg) = serde_json::from_slice::<NetworkMessage>(&message.data) {
                                let _ = self.blockchain_tx.send(msg.message_type).await;
                            }
                        }
                        SwarmEvent::NewListenAddr { address, .. } => {
                            println!("Listening on {}", address);
                        }
                        _ => {}
                    }
                }
            }
        }
    }
}