use ed25519_dalek::{SigningKey, VerifyingKey};
use serde::{Deserialize, Serialize};
use rand;
use std::fs;
use std::path::Path;

#[derive(Debug, Serialize, Deserialize)]
pub struct Wallet {
    pub address: String,
    #[serde(skip)]
    pub private_key: Option<SigningKey>,
    pub public_key: String,
    pub atproto_did: Option<String>,
}

impl Wallet {
    pub fn new(atproto_did: Option<String>) -> Self {
        let private_key = SigningKey::from_bytes(&rand::random::<[u8; 32]>());
        let public_key = private_key.verifying_key();
        
        Wallet {
            address: hex::encode(public_key.to_bytes()),
            private_key: Some(private_key),
            public_key: hex::encode(public_key.to_bytes()),
            atproto_did,
        }
    }

    pub fn from_private_key(private_key_hex: &str, atproto_did: Option<String>) -> Result<Self, String> {
        let private_key_bytes = hex::decode(private_key_hex)
            .map_err(|_| "Invalid hex string")?;
        
        let private_key = SigningKey::from_bytes(&private_key_bytes.try_into().unwrap());
        let public_key = private_key.verifying_key();
        
        Ok(Wallet {
            address: hex::encode(public_key.to_bytes()),
            private_key: Some(private_key),
            public_key: hex::encode(public_key.to_bytes()),
            atproto_did,
        })
    }

    pub fn get_private_key_hex(&self) -> Option<String> {
        self.private_key.as_ref().map(|k| hex::encode(k.to_bytes()))
    }

    pub fn get_verifying_key(&self) -> Result<VerifyingKey, String> {
        let bytes = hex::decode(&self.public_key)
            .map_err(|_| "Invalid public key hex")?;
        let bytes_array: [u8; 32] = bytes.try_into()
            .map_err(|_| "Invalid public key length")?;
        Ok(VerifyingKey::from_bytes(&bytes_array)
            .map_err(|_| "Invalid public key")?)
    }
    
    pub fn export_to_file(&self, path: &str) -> Result<(), String> {
        let wallet_data = WalletData {
            address: self.address.clone(),
            private_key: self.get_private_key_hex(),
            public_key: self.public_key.clone(),
            atproto_did: self.atproto_did.clone(),
        };
        
        let json = serde_json::to_string_pretty(&wallet_data)
            .map_err(|e| format!("Failed to serialize wallet: {}", e))?;
            
        fs::write(path, json)
            .map_err(|e| format!("Failed to write wallet file: {}", e))?;
            
        Ok(())
    }
    
    pub fn import_from_file(path: &str) -> Result<Self, String> {
        if !Path::new(path).exists() {
            return Err("Wallet file does not exist".to_string());
        }
        
        let json = fs::read_to_string(path)
            .map_err(|e| format!("Failed to read wallet file: {}", e))?;
            
        let wallet_data: WalletData = serde_json::from_str(&json)
            .map_err(|e| format!("Failed to parse wallet file: {}", e))?;
            
        if let Some(private_key) = wallet_data.private_key {
            Self::from_private_key(&private_key, wallet_data.atproto_did)
        } else {
            Err("No private key found in wallet file".to_string())
        }
    }
}

#[derive(Debug, Serialize, Deserialize)]
struct WalletData {
    address: String,
    private_key: Option<String>,
    public_key: String,
    atproto_did: Option<String>,
}